import glm
import numpy as np
from OpenGL.GL import *


class BoxDrawer():

    def __init__(self, program):

        self.program = program

        glUseProgram(self.program)
        self.aPositionLocation = glGetAttribLocation(self.program, "aPosition")
        self.aTexCoordLocation = glGetAttribLocation(self.program, "aTexCoord")
        self.uMvpMatrixLocation = glGetUniformLocation(self.program, "uMvpMatrix")
        uSamplerLocation = glGetUniformLocation(self.program, "uSampler")
        glUniform1i(uSamplerLocation, 0)

        self.initVertexBuffers()

    def initVertexBuffers(self):

        vertPositions = np.array([
            -0.5, -0.5, 0.5,
            -0.5, 0.5, 0.5,
            -0.5, 0.5, -0.5,
            -0.5, 0.5, 0.5,
            0.5, 0.5, 0.5,
            0.5, 0.5, -0.5,
            0.5, 0.5, 0.5,
            0.5, -0.5, 0.5,
            0.5, -0.5, -0.5,
            0.5, -0.5, 0.5,
            -0.5, -0.5, 0.5,
            -0.5, -0.5, -0.5,
            -0.5, -0.5, -0.5,
            -0.5, 0.5, -0.5,
            0.5, 0.5, -0.5,
            0.5, -0.5, 0.5,
            0.5, 0.5, 0.5,
            -0.5, 0.5, 0.5,
            -0.5, -0.5, -0.5,
            -0.5, -0.5, 0.5,
            -0.5, 0.5, -0.5,
            -0.5, 0.5, -0.5,
            -0.5, 0.5, 0.5,
            0.5, 0.5, -0.5,
            0.5, 0.5, -0.5,
            0.5, 0.5, 0.5,
            0.5, -0.5, -0.5,
            0.5, -0.5, -0.5,
            0.5, -0.5, 0.5,
            -0.5, -0.5, -0.5,
            0.5, -0.5, -0.5,
            -0.5, -0.5, -0.5,
            0.5, 0.5, -0.5,
            -0.5, -0.5, 0.5,
            0.5, -0.5, 0.5,
            -0.5, 0.5, 0.5
        ], dtype=np.float32)
        self.vertPosBuffer = glGenBuffers(1)
        glBindBuffer(GL_ARRAY_BUFFER, self.vertPosBuffer)
        glBufferData(GL_ARRAY_BUFFER, len(vertPositions) * 4,
            vertPositions, GL_STATIC_DRAW)

        texCoords = np.array([
            1, 1,
            0, 1,
            0, 0,
            1, 1,
            0, 1,
            0, 0,
            1, 1,
            0, 1,
            0, 0,
            1, 1,
            0, 1,
            0, 0,
            0, 1,
            0, 0,
            1, 0,
            1, 0,
            1, 1,
            0, 1,
            1, 0,
            1, 1,
            0, 0,
            1, 0,
            1, 1,
            0, 0,
            1, 0,
            1, 1,
            0, 0,
            1, 0,
            1, 1,
            0, 0,
            1, 1,
            0, 1,
            1, 0,
            0, 0,
            1, 0,
            0, 1
        ], dtype=np.float32)
        self.texCoordBuffer = glGenBuffers(1)
        glBindBuffer(GL_ARRAY_BUFFER, self.texCoordBuffer)
        glBufferData(GL_ARRAY_BUFFER, len(texCoords) * 4,
            texCoords, GL_STATIC_DRAW)

    def bind(self):
        glUseProgram(self.program)

        glBindBuffer(GL_ARRAY_BUFFER, self.vertPosBuffer)
        glVertexAttribPointer(self.aPositionLocation, 3, GL_FLOAT, GL_FALSE, 0, ctypes.c_void_p(0))
        glEnableVertexAttribArray(self.aPositionLocation)

        glBindBuffer(GL_ARRAY_BUFFER, self.texCoordBuffer)
        glVertexAttribPointer(self.aTexCoordLocation, 2, GL_FLOAT, GL_FALSE, 0, ctypes.c_void_p(0))
        glEnableVertexAttribArray(self.aTexCoordLocation)

    def draw(self, projViewMatrix, position, size, rotation, texture):
        self.bind()
        glBindTexture(GL_TEXTURE_2D, texture)

        # Create a model matrix, that is, a matrix combining the
        # translation matrix, rotation matrix, and the scale matrix
        modelMatrix = glm.translate(glm.mat4(1), position)
        rotationMatrix = glm.mat4_cast(rotation)
        modelMatrix = modelMatrix * rotationMatrix
        modelMatrix = glm.scale(modelMatrix, size)

        # Combine projView matrix and model matrix into one MVP matrix
        mvpMatrix = projViewMatrix * modelMatrix

        # Send MVP matrix to the vertex shader
        glUniformMatrix4fv(self.uMvpMatrixLocation, 1, GL_FALSE, glm.value_ptr(mvpMatrix))

        # Draw a rectangle
        glDrawArrays(GL_TRIANGLES, 0, 36)
