from panda3d.bullet import BulletBoxShape, BulletRigidBodyNode
from panda3d.core import Point3, Quat, TransformState, Vec3


class Collider():

    def __init__(self, name, position, rotation, size, mass, world):

        self.position = position
        self.rotation = rotation
        self.size = size

        shape = BulletBoxShape(Vec3(size.x / 2, size.y / 2, size.z / 2))
        self.node = BulletRigidBodyNode(name)
        self.node.addShape(shape)
        self.node.setMass(mass)

        p = Point3(self.position.x, self.position.y, self.position.z)
        q = Quat.identQuat()
        s = Vec3(1, 1, 1)

        self.transform = TransformState.make_pos_quat_scale(p, q, s)
        self.node.setTransform(self.transform)

        world.attachRigidBody(self.node)

    def update(self):
        self.position.x = self.node.getTransform().pos.x
        self.position.y = self.node.getTransform().pos.y
        self.position.z = self.node.getTransform().pos.z

        hpr = self.node.getTransform().getHpr()
        quat = Quat()
        quat.setHpr(hpr)

        self.rotation.w = quat.getX()
        self.rotation.x = quat.getY()
        self.rotation.y = quat.getZ()
        self.rotation.z = quat.getW()
